# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2019-07-18 14:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('activity_id', models.PositiveIntegerField(db_index=True, unique=True, verbose_name='Activity ID')),
                ('from_template_activity_slug', models.CharField(max_length=128,
                                                                 verbose_name='Template Activity Slug',
                                                                 null=True, default=None)),
            ],
            options={
                'verbose_name': 'Activity',
                'verbose_name_plural': 'Activities',
            },
        ),
        migrations.CreateModel(
            name='Process',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('process_key', models.CharField(db_index=True,
                                                 max_length=128, unique=True, verbose_name='Process Key')),
                ('internal_key', models.CharField(db_index=True,
                                                  max_length=128, verbose_name='Internal Key')),
                ('process_type_slug', models.CharField(max_length=128, verbose_name='Process Type')),
                ('status', models.CharField(max_length=128, verbose_name='Status')),
            ],
            options={
                'verbose_name': 'Process',
                'verbose_name_plural': 'Processes',
            },
        ),
        migrations.AddField(
            model_name='activity',
            name='process',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='activities', to='activity.Process', verbose_name='Process'),
        ),
    ]
